"""
--------------------------------------------------------------------------
Cylon LEDs
--------------------------------------------------------------------------
License:   
Copyright 2018 Octavo Systems

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors 
may be used to endorse or promote products derived from this software without 
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--------------------------------------------------------------------------

Make the LEDs cylon for the C-SiP Deadbug:
  - USR0 - GPMC_CSN0      - GPIO1_29 - GPIO 61
  - USR1 - GPMC_OEN_REN   - GPIO2_3  - GPIO 67
  - USR2 - GPMC_WEN       - GPIO2_4  - GPIO 68
  - USR3 - GPMC_BEN0_CLE  - GPIO2_5  - GPIO 69

"""
import time

import Adafruit_BBIO.GPIO as GPIO

# ------------------------------------------------------------------------
# Global variables
# ------------------------------------------------------------------------

# LED GPIO values
LED0                         = (1, 29)           # gpio61 
LED1                         = (2, 3)            # gpio67 
LED2                         = (2, 4)            # gpio68 
LED3                         = (2, 5)            # gpio69 

# Time interval
interval                     = 0.1


# ------------------------------------------------------------------------
# GPIO / ADC access library
# ------------------------------------------------------------------------
import os

# Peripheral path
GPIO_BASE_PATH               = "/sys/class/gpio"

# GPIO direction
IN                           = True
OUT                          = False

# GPIO output state
LOW                          = "0"
HIGH                         = "1"

def gpio_setup(gpio, direction, default_value=False):
    """Setup GPIO pin
    
      * Test if GPIO exists; if not create it
      * Set direction
      * Set default value
    """
    gpio_number = str((gpio[0] * 32) + gpio[1])
    path        = "{0}/gpio{1}".format(GPIO_BASE_PATH, gpio_number)
    
    if not os.path.exists(path):
        # "echo {gpio_number} > {GPIO_BASE_PATH}/export"
        print("Create GPIO: {0}".format(gpio_number))
        with open("{0}/export".format(GPIO_BASE_PATH), 'w') as f:
            f.write(gpio_number)
    
    if direction:
        # "echo in > {path}/direction"
        with open("{0}/direction".format(path), 'w') as f:
            f.write("in")
    else:
        # "echo out > {path}/direction"
        with open("{0}/direction".format(path), 'w') as f:
            f.write("out")
        
    if default_value:
        # "echo {default_value} > {path}/value"
        with open("{0}/value".format(path), 'w') as f:
            f.write(default_value)
    
# End def


def gpio_set(gpio, value):
    """Set GPIO ouptut value."""
    gpio_number = str((gpio[0] * 32) + gpio[1])
    path        = "{0}/gpio{1}".format(GPIO_BASE_PATH, gpio_number)
    
    # "echo {value} > {path}/value"
    with open("{0}/value".format(path), 'w') as f:
        f.write(value)

# End def


def gpio_get(gpio):
    """Get GPIO input value."""
    gpio_number = str((gpio[0] * 32) + gpio[1])
    path        = "{0}/gpio{1}".format(GPIO_BASE_PATH, gpio_number)
    
    # "cat {path}/value"
    with open("{0}/value".format(path), 'r') as f:
        out = f.read()
    
    return float(out)

# End def


# ------------------------------------------------------------------------
# Main Tasks
# ------------------------------------------------------------------------

def setup():
    """Setup the hardware components."""
    gpio_setup(LED0, OUT, LOW)
    gpio_setup(LED1, OUT, LOW)
    gpio_setup(LED2, OUT, LOW)
    gpio_setup(LED3, OUT, LOW)
    
# End def


def task():
    """Execute the main program."""

    while(True):
        gpio_set(LED0, HIGH)
        gpio_set(LED1, LOW)
        time.sleep(interval)

        gpio_set(LED1, HIGH)
        gpio_set(LED0, LOW)
        time.sleep(interval)
    
        gpio_set(LED2, HIGH)
        gpio_set(LED1, LOW)
        time.sleep(interval)
    
        gpio_set(LED3, HIGH)
        gpio_set(LED2, LOW)
        time.sleep(interval)
        
        gpio_set(LED2, HIGH)
        gpio_set(LED3, LOW)
        time.sleep(interval)

        gpio_set(LED1, HIGH)
        gpio_set(LED2, LOW)
        time.sleep(interval)    
        
# End def


def cleanup():
    """Cleanup the hardware components."""
    pass
    
# End def


# ------------------------------------------------------------------------
# Main script
# ------------------------------------------------------------------------

if __name__ == '__main__':
    setup()
    
    try:
        task()
    except KeyboardInterrupt:
        pass

    cleanup()
    print("Program Complete.")

